@@ -1,5 +1,5 @@
 /****************************************************************************
- * RRDtool 1.0.37  Copyright Tobias Oetiker, 1997 - 2000
+ * RRDtool 1.0.46  Copyright Tobias Oetiker, 1997 - 2000
  ****************************************************************************
  * gifsize.c  provides the function gifsize which determines the size of a gif
  ****************************************************************************/
@@ -52,7 +52,7 @@ static int ReadColorMap (FILE *fd, int number, unsigned char (*buffer)[256]);
 static int DoExtension (FILE *fd, int label, int *Transparent);
 static int GetDataBlock (FILE *fd, unsigned char *buf);
 
-int ZeroDataBlock;
+int gifsize_ZeroDataBlock;
 
 int
 GifSize(FILE *fd, long *width, long *height)
@@ -68,7 +68,7 @@ GifSize(FILE *fd, long *width, long *height)
        unsigned char   ColorMap[3][MAXCOLORMAPSIZE];
        int             imageCount = 0;
        char            version[4];
-       ZeroDataBlock = FALSE;
+       gifsize_ZeroDataBlock = FALSE;
 
        imageNumber = 1;
        if (! ReadOK(fd,buf,6)) {
@@ -185,7 +185,7 @@ GetDataBlock(FILE *fd, unsigned char *buf)
                return -1;
        }
 
-       ZeroDataBlock = count == 0;
+       gifsize_ZeroDataBlock = count == 0;
 
        if ((count != 0) && (! ReadOK(fd, buf, count))) {
                return -1;
