@@ -1,7 +1,7 @@
 /*****************************************************************************
- * RRDtool 1.0.37  Copyright Tobias Oetiker, 1997 - 2000
+ * RRDtool 1.0.46  Copyright Tobias Oetiker, 1997 - 2000
  *****************************************************************************
- * rrd_create.c  creates new rrds
+ * rrd_create.c  creates new RRDs
  *****************************************************************************/
 
 #include "rrd_tool.h"
@@ -13,7 +13,7 @@ rrd_create(int argc, char **argv)
     rrd_t          rrd;
     long                i,long_tmp;
     time_t             last_up;
-    struct time_value last_up_tv;
+    struct rrd_time_value last_up_tv;
     char *parsetime_error = NULL;
 
     /* init last_up */
