@@ -1,5 +1,5 @@
 /*****************************************************************************
- * RRDtool 1.0.37  Copyright Tobias Oetiker, 1999
+ * RRDtool 1.0.46  Copyright Tobias Oetiker, 1999
  * This code is stolen from rateup (mrtg-2.x) by Dave Rand
  *****************************************************************************
  * diff calculate the difference between two very long integers available as
@@ -24,28 +24,37 @@
  *
  *****************************************************************************/
 
-#include "rrd_tool.h"
-
+#include "rrd_tool.h" 
 double
 rrd_diff(char *a, char *b)
 {
     char res[LAST_DS_LEN+1], *a1, *b1, *r1, *fix;
     int c,x,m;
-    
-    while (!(isdigit((int)*a) || *a==0))
+    char a_neg=0, b_neg=0;
+    double result;
+   
+    while (!(isdigit((int)*a) || *a==0)) {
+        if(*a=='-') 
+	    a_neg = 1;
         a++;
+    }
     fix=a;
     while (isdigit((int)*fix)) 
 	fix++;
     *fix = 0; /* maybe there is some non digit data in the string */ 
-    while (!(isdigit((int)*b) || *b==0))
+    while (!(isdigit((int)*b) || *b==0)) {
+	if(*b=='-') 
+	    b_neg = 1;  
         b++;
+    }
     fix=b;
     while (isdigit((int)*fix)) 
 	fix++;
     *fix = 0; /* maybe there is some non digit data in the string */ 
     if(!isdigit((int)*a) || !isdigit((int)*b))
 	return DNAN;
+    if(a_neg+b_neg == 1) /* can not handle numbers with different signs yet */
+	return DNAN;
     a1 = &a[strlen(a)-1];
     m = max(strlen(a),strlen(b));
     if (m > LAST_DS_LEN) return DNAN; /* result string too short */
@@ -56,13 +65,11 @@ rrd_diff(char *a, char *b)
     r1[1] = 0;  /* Null terminate result */
     c = 0;
     for (x=0; x<m; x++) {
-        if (a1 >= a && b1 >= b) {
-            *r1 = ((*a1 - c) - *b1) + '0';
-        } else if (a1 >= a) {
-            *r1 = (*a1 - c);
-        } else {
-            *r1 = ('0' - *b1 - c) + '0';
-        }
+        /* we want to avoid reading off the edge of the string */
+        char save_a,save_b;
+        save_a = ( a1 >= a) ? *a1 : '0';
+        save_b = ( b1 >= b) ? *b1 : '0';
+        *r1 = save_a - save_b - c + '0';
         if (*r1 < '0') {
             *r1 += 10;
             c=1;
@@ -86,7 +93,12 @@ rrd_diff(char *a, char *b)
                 c=0;
             }
         }
-        return(-atof(res));
+        result = -atof(res);
     } else
-        return(atof(res));
+        result = atof(res);
+
+    if(a_neg+b_neg==2) /* both are negatives, reverse sign */
+        result = -result;
+    
+    return result;
 }                                                       
