@@ -1,5 +1,5 @@
 /*****************************************************************************
- * RRDtool 1.0.37  Copyright Tobias Oetiker, 1997 - 2000
+ * RRDtool 1.0.46  Copyright Tobias Oetiker, 1997 - 2000
  *****************************************************************************
  * rrd_fetch.c  read date from an rrd to use for further processing
  *****************************************************************************
@@ -31,7 +31,7 @@ rrd_fetch(int argc,
     time_t   start_tmp=0, end_tmp=0;
     enum     cf_en cf_idx;
 
-    struct time_value start_tv, end_tv;
+    struct rrd_time_value start_tv, end_tv;
     char     *parsetime_error = NULL;
 
     /* init start and end time */
@@ -144,7 +144,7 @@ rrd_fetch_fn(
     if(rrd_open(filename,&in_file,&rrd, RRD_READONLY)==-1)
 	return(-1);
     
-    /* when was the realy last update of this file ? */
+    /* when was the really last update of this file ? */
 
     if (((*ds_namv) = (char **) malloc(rrd.stat_head->ds_cnt * sizeof(char*)))==NULL){
 	rrd_set_error("malloc fetch ds_namv array");
@@ -179,12 +179,13 @@ rrd_fetch_fn(
 			    * rrd.rra_def[i].row_cnt
 			    * rrd.stat_head->pdp_step));
 
+            /* we need step difference in either full or partial case */
+   	    tmp_step_diff = labs(*step - (rrd.stat_head->pdp_step
+					 * rrd.rra_def[i].pdp_cnt));
 	    full_match = *end -*start;
 	    /* best full match */
 	    if(cal_end >= *end 
 	       && cal_start <= *start){
-		tmp_step_diff = labs(*step - (rrd.stat_head->pdp_step
-					 * rrd.rra_def[i].pdp_cnt));
 		if (first_full || (tmp_step_diff < best_step_diff)){
 		    first_full=0;
 		    best_step_diff = tmp_step_diff;
@@ -198,9 +199,13 @@ rrd_fetch_fn(
 		    tmp_match -= (cal_start-*start);
 		if (cal_end<*end)
 		    tmp_match -= (*end-cal_end);		
-		if (first_part || best_match < tmp_match){
+                if (first_part ||
+                    (best_match < tmp_match) ||
+                    (best_match == tmp_match && 
+                     tmp_step_diff < best_step_diff)){ 
 		    first_part=0;
 		    best_match = tmp_match;
+		    best_step_diff = tmp_step_diff;
 		    best_part_rra =i;
 		} 
 	    }
