@@ -1,5 +1,5 @@
 /*****************************************************************************
- * RRDtool 1.0.37  Copyright Tobias Oetiker, 1999
+ * RRDtool 1.0.46  Copyright Tobias Oetiker, 1999
  *****************************************************************************
  * rrd_format.c  RRD Database Format helper functions
  *****************************************************************************
@@ -10,7 +10,7 @@
  *
  * Revision 1.3  1998/03/08 12:35:11  oetiker
  * checkpointing things because the current setup seems to work
- * according to the things said in the manpages
+ * according to the things said in the man pages
  *
  * Revision 1.2  1998/02/26 22:58:22  oetiker
  * fixed define
@@ -32,7 +32,7 @@ enum dst_en dst_conv(char *string)
     converter(ABSOLUTE,DST_ABSOLUTE)
     converter(GAUGE,DST_GAUGE)
     converter(DERIVE,DST_DERIVE)
-    rrd_set_error("unknown date aquisition function '%s'",string);
+    rrd_set_error("unknown date acquisition function '%s'",string);
     return(-1);
 }
 
