@@ -1,6 +1,6 @@
 
 /****************************************************************************
- * RRDtool 1.0.40  Copyright Tobias Oetiker, 1997 - 2000
+ * RRDtool 1.0.46  Copyright Tobias Oetiker, 1997 - 2000
  ****************************************************************************
  * rrd__graph.c  make creates ne rrds
  ****************************************************************************/
@@ -8,8 +8,15 @@
 #include "rrd_tool.h"
 #include "rrd_graph.h"
 #include <gd.h>
+
+#ifdef LATIN2_ENABLE
+#include <gdlucidan10l2.h>
+#include <gdlucidab12l2.h>
+#else
 #include <gdlucidan10.h>
 #include <gdlucidab12.h>
+#endif
+
 #include <sys/stat.h>
 #ifdef WIN32
 #include <io.h>
@@ -23,9 +30,14 @@
 #ifdef HAVE_LOCALE_H
 #include <locale.h>
 #endif
-    
+
+#ifdef LATIN2_ENABLE
+#define SmallFont gdLucidaNormal10l2
+#define LargeFont gdLucidaBold12l2
+#else    
 #define SmallFont gdLucidaNormal10
 #define LargeFont gdLucidaBold12
+#endif
 
 /* #define DEBUG */
 
@@ -163,9 +175,19 @@ ytr(image_desc_t *im, double value){
     } 
 }
 
-   
-    
+/* allocate colors with re-use */
 
+int color_allocate(gdImagePtr im, int r, int g, int b) { 
+
+	int color;
+
+	if ((color = gdImageColorExact(im, r, g, b)) != -1) {
+		return(color);
+	}
+
+	return(gdImageColorAllocate(im, r, g, b));
+}
+    
 
 /* conversion function for symbolic entry names */
 
@@ -508,7 +530,7 @@ reduce_data(
 
 
 /* Sanity check: row_cnt should be multiple of reduce_factor */
-/* if this gets triggered, something is REALY WRONG ... we die immediately */
+/* if this gets triggered, something is REALLY WRONG ... we die immediately */
 
     if (row_cnt%reduce_factor) {
 	printf("SANITY CHECK: %lu rows cannot be reduced by %i \n",
@@ -636,7 +658,7 @@ data_fetch( image_desc_t *im )
 	    }
 	}
 	
-        /* lets see if the required data source is realy there */
+        /* lets see if the required data source is really there */
 	for(ii=0;ii<im->gdes[i].ds_cnt;ii++){
 	    if(strcmp(im->gdes[i].ds_namv[ii],im->gdes[i].ds_nam) == 0){
 		im->gdes[i].ds=ii; }
@@ -694,22 +716,33 @@ rpnp_t *
 str2rpn(image_desc_t *im,char *expr){
     int pos=0;
     long steps=-1;    
-    rpnp_t  *rpnp;
+    rpnp_t  *rpnp=NULL;
     char vname[30];
-
-    rpnp=NULL;
+    char *tmp_locale;
+    int ret=0;
 
     while(*expr){
+
 	if ((rpnp = (rpnp_t *) rrd_realloc(rpnp, (++steps + 2)* 
 				       sizeof(rpnp_t)))==NULL){
 	    return NULL;
 	}
-
+#if 0
 	else if((sscanf(expr,"%lf%n",&rpnp[steps].val,&pos) == 1) 
 	        && (expr[pos] == ',')){
  	    rpnp[steps].op = OP_NUMBER;
 	    expr+=pos;
 	} 
+#endif
+	tmp_locale = setlocale(LC_NUMERIC, "C");
+	ret=sscanf(expr,"%lf%n",&rpnp[steps].val,&pos);
+	if (tmp_locale != NULL)
+		setlocale(LC_NUMERIC, tmp_locale);		
+	
+ 	if ((ret == 1) && (expr[pos] == ',')){
+		rpnp[steps].op = OP_NUMBER;
+		expr+=pos;
+	}
 	
 #define match_op(VV,VVV) \
         else if (strncmp(expr, #VVV, strlen(#VVV))==0 && \
@@ -967,7 +1000,7 @@ data_calc( image_desc_t *im){
 			} else {
                             int rpi_ptr = im->gdes[gdi].rpnp[rpi].ptr;
                             int ds_cnt = im->gdes[gdi].rpnp[rpi].ds_cnt;
-                            stack[++stptr] = im->gdes[rpi_ptr].data[dataidx*ds_cnt+rpi_ptr];
+                            stack[++stptr] = im->gdes[rpi_ptr].data[dataidx*ds_cnt+im->gdes[rpi_ptr].ds];
 			}
 		    break;
 		case OP_UNKN:
@@ -1334,7 +1367,7 @@ data_proc( image_desc_t *im ){
 	}
     }
 
-    /* if min or max have not been asigned a value this is because
+    /* if min or max have not been assigned a value this is because
        there was no data in the graph ... this is not good ...
        lets set these to dummy values then ... */
 
@@ -1354,6 +1387,10 @@ data_proc( image_desc_t *im ){
 	else
 	    im->maxval = maxval;
     }
+    /* make sure min is smaller than max */
+    if (im->minval > im->maxval) {
+       im->minval = 0.99 * im->maxval;
+    }
     /* make sure min and max are not equal */
     if (im->minval == im->maxval) {
       im->maxval *= 1.01; 
@@ -1454,7 +1491,7 @@ find_next_time(
 	    tm.tm_year += basestep;	
 	}
 	madetime = mktime(&tm);
-    } while (madetime == -1); /* this is necessary to skip impssible times
+    } while (madetime == -1); /* this is necessary to skip impossible times
 				 like the daylight saving time skips */
     return madetime;
 	  
@@ -1472,13 +1509,12 @@ void gator( gdImagePtr gif, int x, int y){
 
 /* the fact that the text of what gets put into the graph is not
    visible in the function, has lead some to think this is for
-   obfuscation reasons. While this is a nice side effect (I addmit),
+   obfuscation reasons. While this is a nice side effect (I admit),
    it is not the prime reason. The prime reason is, that the font
    used, is so small, that I had to hand edit the characters to ensure
    readability. I could thus not use the normal gd functions to write,
    but had to embed a slightly compressed bitmap version into the code. 
 */
-
     int li[]={0,0,1, 0,4,5, 0,8,9, 0,12,14, 0,17,17, 0,21,21, 
 	      0,24,24, 0,34,34, 0,40,42, 0,45,45, 0,48,49, 0,52,54, 
 	      0,61,61, 0,64,66, 0,68,70, 0,72,74, 0,76,76, 0,78,78, 
@@ -1647,23 +1683,31 @@ leg_place(image_desc_t *im)
     int   leg_c = 0;
     int   leg_x = border, leg_y = im->ygif;
     int   leg_cc;
-    int   glue = 0;
+    int   glue = 0, useglue = 1;
     int   i,ii, mark = 0;
     char  prt_fctn; /*special printfunctions */
     int  *legspace;
 
-  if( !(im->extra_flags & NOLEGEND) ) {
-    if ((legspace = malloc(im->gdes_c*sizeof(int)))==NULL){
-       rrd_set_error("malloc for legspace");
-       return -1;
-    }
+    if( !(im->extra_flags & NOLEGEND) ) {
+      if ((legspace = malloc(im->gdes_c*sizeof(int)))==NULL){
+         rrd_set_error("malloc for legspace");
+         return -1;
+      }
 
     for(i=0;i<im->gdes_c;i++){
 	fill_last = fill;
 
+	if (im->gdes[i].gf == GF_HRULE &&
+	    (im->gdes[i].yrule < im->minval || im->gdes[i].yrule > im->maxval))
+	    im->gdes[i].legend[0] = '\0';
+
+	if (im->gdes[i].gf == GF_VRULE &&
+	    (im->gdes[i].xrule < im->start || im->gdes[i].xrule > im->end))
+	    im->gdes[i].legend[0] = '\0';
+
 	leg_cc = strlen(im->gdes[i].legend);
 	
-	/* is there a controle code ant the end of the legend string ? */ 
+	/* is there a control code ant the end of the legend string ? */ 
 	if (leg_cc >= 2 && im->gdes[i].legend[leg_cc-2] == '\\') {
 	    prt_fctn = im->gdes[i].legend[leg_cc-1];
 	    leg_cc -= 2;
@@ -1695,9 +1739,13 @@ leg_place(image_desc_t *im)
 	   legspace[i]=0;
 	}
         /* who said there was a special tag ... ?*/
-	if (prt_fctn=='g') {    
+        if (prt_fctn=='J') {    
+           useglue = 0;
+           prt_fctn = '\0';
+        } else if (prt_fctn=='g') {    
 	   prt_fctn = '\0';
-	}
+	} 
+
 	if (prt_fctn == '\0') {
 	    if (i == im->gdes_c -1 ) prt_fctn ='l';
 	    
@@ -1719,9 +1767,9 @@ leg_place(image_desc_t *im)
 
 	if (prt_fctn != '\0'){
 	    leg_x = border;
-	    if (leg_c >= 2 && prt_fctn == 'j') {
-		glue = (im->xgif - fill - 2* border) / (leg_c-1);
-		/* if (glue > 2 * SmallFont->w) glue = 0; */
+	    if (leg_c >= 2 && prt_fctn == 'j' && useglue) {
+                glue = (im->xgif - fill - 2* border) / (leg_c-1);
+	        /* if (glue > 2 * SmallFont->w) glue = 0; */
 	    } else {
 		glue = 0;
 	    }
@@ -1849,7 +1897,6 @@ horizontal_grid(gdImagePtr gif, image_desc_t   *im)
 	gridstep = im->ygridstep;
 	labfact = im->ylabfact;
     }
-    
     polyPoints[0].x=im->xorigin;
     polyPoints[1].x=im->xorigin+im->xsize;
 
@@ -1863,7 +1910,7 @@ horizontal_grid(gdImagePtr gif, image_desc_t   *im)
 	    if(i % labfact == 0){
 		if (i==0 || im->symbol == ' ') {
 		    if(scaledstep < 1){
-			if(im->extra_flags & ALTYGRID) {
+			if(i!=0 && im->extra_flags & ALTYGRID) {
 			    sprintf(graph_label,labfmt,scaledstep*i);
 			}
 			else {
@@ -1892,15 +1939,18 @@ horizontal_grid(gdImagePtr gif, image_desc_t   *im)
 			    polyPoints[0].x+2,polyPoints[0].y,graph_col[GRC_MGRID].i);
 		gdImageLine(gif, polyPoints[1].x-2,polyPoints[0].y,
 			    polyPoints[1].x+2,polyPoints[0].y,graph_col[GRC_MGRID].i);		    
-	    } else {		
+
+                gdImageLine(gif, polyPoints[0].x,polyPoints[0].y,
+                            polyPoints[1].x,polyPoints[0].y,gdStyled);
+	    } else if ( !(im->extra_flags & NOMINOR) ) {		
 		gdImageSetStyle(gif, styleMinor, 2);
 		gdImageLine(gif, polyPoints[0].x-1,polyPoints[0].y,
 			    polyPoints[0].x+1,polyPoints[0].y,graph_col[GRC_GRID].i);
 		gdImageLine(gif, polyPoints[1].x-1,polyPoints[0].y,
 			    polyPoints[1].x+1,polyPoints[0].y,graph_col[GRC_GRID].i);		    
+                gdImageLine(gif, polyPoints[0].x,polyPoints[0].y,
+                            polyPoints[1].x,polyPoints[0].y,gdStyled);
 	    }	    
-	    gdImageLine(gif, polyPoints[0].x,polyPoints[0].y,
-			polyPoints[1].x,polyPoints[0].y,gdStyled);
 	}	
     } 
 /*    if(im->minval * im->maxval < 0){
@@ -1960,7 +2010,7 @@ horizontal_mrtg_grid(gdImagePtr gif, image_desc_t *im)
  return 1;
 }
 
-/* logaritmic horizontal grid */
+/* logarithmic horizontal grid */
 int
 horizontal_log_grid(gdImagePtr gif, image_desc_t   *im)
 {
@@ -2090,6 +2140,8 @@ vertical_grid(
     polyPoints[1].y = im->yorigin-im->ysize;
 
     /* paint the minor grid */
+    if ( !(im->extra_flags & NOMINOR ))
+    {
     for(ti = find_first_time(im->start,
 			    im->xlab_user.gridtm,
 			    im->xlab_user.gridst);
@@ -2111,6 +2163,7 @@ vertical_grid(
 	gdImageLine(gif, polyPoints[0].x,polyPoints[1].y-1,
 		    polyPoints[0].x,polyPoints[1].y+1,graph_col[GRC_GRID].i);
     }
+    }
 
     /* paint the major grid */
     for(ti = find_first_time(im->start,
@@ -2323,13 +2376,13 @@ MkLineBrush(image_desc_t *im,long cosel, enum gf_en typsel){
     return NULL;
   }
 
-  gdImageColorTransparent(brush, 
-			  gdImageColorAllocate(brush, 0, 0, 0));
 
   pen = gdImageColorAllocate(brush, 
 			     im->gdes[cosel].col.red,
 			     im->gdes[cosel].col.green,
 			     im->gdes[cosel].col.blue);
+
+  gdImageColorTransparent(brush, gdImageColorAllocate(brush, 0, 0, 0));
     
   switch (typsel){
   case GF_LINE1:
@@ -2367,7 +2420,7 @@ copyImage(gdImagePtr gif, char *image, int copy_white)
      {c2 = gdImageGetPixel(img,i1,i2);	/* get pixel */
       if (copy_white || img->red[c2] != 255 || img->green[c2] != 255 || img->blue[c2] != 255)
        {if ((c1 = gdImageColorExact(gif,img->red[c2],img->green[c2],img->blue[c2])) < 0)
-         if ((c1 = gdImageColorAllocate(gif,img->red[c2],img->green[c2],img->blue[c2])) < 0)
+         if ((c1 = color_allocate(gif,img->red[c2],img->green[c2],img->blue[c2])) < 0)
           c1 = gdImageColorClosest(gif,img->red[c2],img->green[c2],img->blue[c2]);
         gdImageSetPixel(gif,i1,i2,c1);
        }
@@ -2463,8 +2516,11 @@ graph_paint(image_desc_t *im, char ***calcpr)
        draw labels and other things outside the graph area */
 
 
-    im->xorigin = 10 + 9 * SmallFont->w+SmallFont->h;
-    xtr(im,0); 
+    im->xorigin = 10 + im->unitslength * SmallFont->w;
+    xtr(im,0);
+
+    if(im->ylegend[0] != '\0')
+        im->xorigin += SmallFont->h;
 
     im->yorigin = 14 + im->ysize;
     ytr(im,DNAN);
@@ -2489,13 +2545,13 @@ graph_paint(image_desc_t *im, char ***calcpr)
 	/* check for user override values */
 	if(im->graph_col[i].red != -1)
 	    graph_col[i].i = 
-		gdImageColorAllocate( gif,
+		color_allocate( gif,
 				      im->graph_col[i].red, 
 				      im->graph_col[i].green, 
 				      im->graph_col[i].blue);
 	else
 	    graph_col[i].i = 
-		gdImageColorAllocate( gif,
+		color_allocate( gif,
 				      graph_col[i].red, 
 				      graph_col[i].green, 
 				      graph_col[i].blue);
@@ -2506,7 +2562,7 @@ graph_paint(image_desc_t *im, char ***calcpr)
 	/* only for elements which have a color defined */
 	if (im->gdes[i].col.red != -1)
 	    im->gdes[i].col.i = 
-		gdImageColorAllocate(gif,
+		color_allocate(gif,
 				     im->gdes[i].col.red,
 				     im->gdes[i].col.green,
 				     im->gdes[i].col.blue);
@@ -2700,7 +2756,7 @@ gdes_alloc(image_desc_t *im){
 
     long def_step = (im->end-im->start)/im->xsize;
     
-    if (im->step > def_step) /* step can be increassed ... no decreassed */
+    if (im->step > def_step) /* step can be increased ... no decreased */
       def_step = im->step;
 
     im->gdes_c++;
@@ -2768,7 +2824,7 @@ rrd_graph(int argc, char **argv, char ***prdata, int *xsize, int *ysize)
     unsigned int            col_red,col_green,col_blue;
     long           scancount;
     int linepass = 0; /* stack can only follow directly after LINE* AREA or STACK */    
-    struct time_value start_tv, end_tv;
+    struct rrd_time_value start_tv, end_tv;
     char *parsetime_error = NULL;
     int stroff;    
 
@@ -2797,6 +2853,7 @@ rrd_graph(int argc, char **argv, char ***prdata, int *xsize, int *ysize)
     im.maxval = DNAN;    
     im.interlaced = 0;
     im.unitsexponent= 9999;
+    im.unitslength= 9;
     im.extra_flags= 0;
     im.rigid = 0;
     im.imginfo = NULL;
@@ -2841,12 +2898,14 @@ im.unit = 0; /* do not display unit if not set */
 	    {"imgformat",  required_argument, 0,  'a'},
 	    {"lazy",       no_argument,       0,  'z'},
 	    {"no-legend",  no_argument,       0,  'g'},
-	    {"alt-y-grid", no_argument,       0,   257 },
-	    {"alt-autoscale", no_argument,    0,   258 },
-	    {"alt-autoscale-max", no_argument,    0,   259 },
-	    {"units-exponent",required_argument, 0,  260},
-	    {"step",       required_argument, 0,   261},
-	    {"alt-y-mrtg", no_argument,       0,   262 },
+	    {"alt-y-grid", no_argument,       0,   'Y'},
+            {"no-minor",   no_argument,       0,   'N'},
+ 	    {"alt-autoscale", no_argument,    0,   'A'},
+	    {"alt-autoscale-max", no_argument,    0,   'M'},
+	    {"units-exponent",required_argument, 0,  'X'},
+	    {"units-length",required_argument, 0,  'L'},
+	    {"step",       required_argument, 0,  'S'},
+	    {"alt-y-mrtg", no_argument,       0,  'R'},
 	    {"background", required_argument, 0,  'B'},
 	    {"overlay",    required_argument, 0,  'O'},
 	    {"unit",       required_argument, 0,  'U'},
@@ -2856,32 +2915,38 @@ im.unit = 0; /* do not display unit if not set */
 
 	
 	opt = getopt_long(argc, argv, 
-			  "s:e:x:y:v:w:h:iu:l:rb:oc:t:f:a:zgB:O:U:",
+			  "s:e:x:y:v:w:h:iu:l:rb:oc:t:f:a:zgYAMXL:S:RB:O:U:",
 			  long_options, &option_index);
 
 	if (opt == EOF)
 	    break;
 	
 	switch(opt) {
-	case 257:
+	case 'Y':
 	    im.extra_flags |= ALTYGRID;
 	    break;
-	case 258:
+        case 'N':
+            im.extra_flags |= NOMINOR;
+            break;
+	case 'A':
 	    im.extra_flags |= ALTAUTOSCALE;
 	    break;
-	case 259:
+	case 'M':
 	    im.extra_flags |= ALTAUTOSCALE_MAX;
 	    break;
 	case 'g':
 	    im.extra_flags |= NOLEGEND;
 	    break;
-	case 260:
+	case 'X':
 	    im.unitsexponent = atoi(optarg);
 	    break;
-	case 261:
+	case 'L':
+	    im.unitslength = atoi(optarg);
+	    break;
+	case 'S':
 	    im.step =  atoi(optarg);
 	    break;
-	case 262:
+	case 'R':
 	    im.extra_flags |= ALTYMRTG;
 	    break;
 	case 's':
@@ -3358,9 +3423,9 @@ int bad_format(char *fmt) {
                  if (*ptr == '.') ptr++;
                  while (*ptr >= '0' && *ptr <= '9') ptr++;
      
-                 /* Either 'le' or 'lf' must follow here */
+                 /* Either 'le', 'lf' or 'lg' must follow here */
                  if (*ptr++ != 'l') return 1;
-                 if (*ptr == 'e' || *ptr == 'f') ptr++;
+                 if (*ptr == 'e' || *ptr == 'f' || *ptr == 'g') ptr++;
                  else return 1;
                  n++;
              }
