@@ -1,5 +1,5 @@
 /*****************************************************************************
- * RRDtool 1.0.37  Copyright Tobias Oetiker, 1997 - 2000
+ * RRDtool 1.0.46  Copyright Tobias Oetiker, 1997 - 2000
  *****************************************************************************
  * rrd_restore.c  creates new rrd from data dumped by rrd_dump.c
  *****************************************************************************/
@@ -16,7 +16,7 @@ int read_tag(char **, char *, char *, void *);
 int xml2rrd(char*, rrd_t*, char);
 int rrd_write(char *, rrd_t *);
 
-/* convert all ocurances of <BlaBlaBla> to <blablabla> */
+/* convert all occurrences of <BlaBlaBla> to <blablabla> */
 
 void xml_lc(char* buf){
   int intag=0;
@@ -168,12 +168,12 @@ int xml2rrd(char* buf, rrd_t* rrd, char rc){
   while (eat_tag(&ptr2,"rra") == 1){
       rrd->stat_head->rra_cnt++;
 
-      /* alocate and reset rra definition areas */
+      /* allocate and reset rra definition areas */
       if((rrd->rra_def = rrd_realloc(rrd->rra_def,rrd->stat_head->rra_cnt*sizeof(rra_def_t)))==NULL){
 	  rrd_set_error("allocating rra_def"); return -1; }      
       memset(&(rrd->rra_def[rrd->stat_head->rra_cnt-1]), 0, sizeof(rra_def_t));
 
-      /* alocate and reset consolidation point areas */
+      /* allocate and reset consolidation point areas */
       if((rrd->cdp_prep = rrd_realloc(rrd->cdp_prep,
 				  rrd->stat_head->rra_cnt
 				  *rrd->stat_head->ds_cnt*sizeof(cdp_prep_t)))==NULL){
@@ -278,12 +278,11 @@ rrd_write(char *file_name, rrd_t *rrd)
     FILE             *rrd_file=NULL;
 
     if (strcmp("-",file_name)==0){
-      *rrd_file= *stdout;
+      rrd_file= stdout;
     } else {
       int fd = open(file_name,O_RDWR|O_CREAT|O_EXCL,0666);
       if (fd == -1 || (rrd_file = fdopen(fd,"wb")) == NULL) {
 	rrd_set_error("creating '%s': %s",file_name,strerror(errno));
-	rrd_free(rrd);
         if (fd != -1)
           close(fd);
 	return(-1);
