@@ -1,5 +1,5 @@
 /*****************************************************************************
- * RRDtool 1.0.37  Copyright Tobias Oetiker, 1997 - 2000
+ * RRDtool 1.0.46  Copyright Tobias Oetiker, 1997 - 2000
  *****************************************************************************
  * rrd_update.c  RRD Update Function
  *****************************************************************************
@@ -32,7 +32,7 @@ int
 main(int argc, char **argv){
         rrd_update(argc,argv);
         if (rrd_test_error()) {
-                printf("RRDtool 1.0.37  Copyright 1997-2000 by Tobias Oetiker <tobi@oetiker.ch>\n\n"
+                printf("RRDtool 1.0.46  Copyright 1997-2000 by Tobias Oetiker <tobi@oetiker.ch>\n\n"
                         "Usage: rrdupdate filename\n"
                         "\t\t\t[--template|-t ds-name:ds-name:...]\n"
                         "\t\t\ttime|N:value[:value...]\n\n"
@@ -95,6 +95,7 @@ rrd_update(int argc, char **argv)
     char             **updvals;
     int              wrote_to_file = 0;
     char             *template = NULL;          
+    char             *endptr; /* used in the conversion */
 
 
     while (1) {
@@ -118,7 +119,6 @@ rrd_update(int argc, char **argv)
 
 	case '?':
 	    rrd_set_error("unknown option '%s'",argv[optind-1]);
-            rrd_free(&rrd);
 	    return(-1);
 	}
     }
@@ -137,10 +137,10 @@ rrd_update(int argc, char **argv)
 
         When a file is opened with udpate mode ('+' as the second
         or third character in the ... list of mode argument
-        variables), both input and ouptut may be performed on the
+        variables), both input and output may be performed on the
         associated stream.  However, ...  input may not be directly
         followed by output without an intervening call to a file
-        positioning function, unless the input oepration encounters
+        positioning function, unless the input operation encounters
         end-of-file. */
     fseek(rrd_file, 0, SEEK_CUR);
 
@@ -239,7 +239,7 @@ rrd_update(int argc, char **argv)
 	char *stepper = malloc((strlen(argv[arg_i])+1)*sizeof(char));
         char *step_start = stepper;
         if (stepper == NULL){
-                rrd_set_error("faild duplication argv entry");
+                rrd_set_error("failed duplication argv entry");
                 free(updvals);
                 free(pdp_temp);  
                 free(tmpl_idx);
@@ -275,7 +275,18 @@ rrd_update(int argc, char **argv)
 	if (strcmp(updvals[0],"N")==0){
 	    current_time = time(NULL);
 	} else {
-	    current_time = atol(updvals[0]);
+            errno = 0;
+	    current_time = strtol(updvals[0],&endptr,10);
+            if (errno > 0){
+                rrd_set_error("converting  '%s' to long: %s",updvals[0],strerror(errno));
+                free(step_start);
+                break;
+            };
+            if (endptr[0] != '\0'){
+                rrd_set_error("conversion of '%s' to long not complete: tail '%s'",updvals[0],endptr);
+                free(step_start);
+                break;
+            }
 	}
 	
 	if(current_time <= rrd.live_head->last_up){
@@ -340,7 +351,7 @@ rrd_update(int argc, char **argv)
 	       rrd.ds_def[i].par[DS_mrhb_cnt].u_cnt >= interval) {
 	       double rate = DNAN;
 	       /* the data source type defines how to process the data */
-		/* pdp_temp contains rate * time ... eg the bytes
+		/* pdp_temp contains rate * time ... e.g. the bytes
 		 * transferred during the interval. Doing it this way saves
 		 * a lot of math operations */
 		
@@ -349,9 +360,18 @@ rrd_update(int argc, char **argv)
 		case DST_COUNTER:
 		case DST_DERIVE:
 		    if(rrd.pdp_prep[i].last_ds[0] != 'U'){
+       		       for(ii=0;updvals[i+1][ii] != '\0';ii++){
+       		            if(updvals[i+1][ii] < '0' || updvals[i+1][ii] > '9' || (ii==0 && updvals[i+1][ii] == '-')){
+                                 rrd_set_error("not a simple integer: '%s'",updvals[i+1]);
+                                 break;
+                            }
+      		       }
+             	       if (rrd_test_error()){
+	        	    break;
+        	       }
 		       pdp_new[i]= rrd_diff(updvals[i+1],rrd.pdp_prep[i].last_ds);
 		       if(dst_idx == DST_COUNTER) {
-			  /* simple overflow catcher sugestet by andres kroonmaa */
+			  /* simple overflow catcher suggested by andres Kroonmaa */
 			  /* this will fail terribly for non 32 or 64 bit counters ... */
 			  /* are there any others in SNMP land ? */
 			  if (pdp_new[i] < (double)0.0 ) 
@@ -366,11 +386,29 @@ rrd_update(int argc, char **argv)
 		   }
 		   break;
 		case DST_ABSOLUTE:
-		    pdp_new[i]= atof(updvals[i+1]);
+                    errno = 0;
+	            pdp_new[i] = strtod(updvals[i+1],&endptr);
+                    if (errno > 0){
+                        rrd_set_error("converting  '%s' to float: %s",updvals[i+1],strerror(errno));
+                        break;
+                    };
+                    if (endptr[0] != '\0'){
+                        rrd_set_error("conversion of '%s' to float not complete: tail '%s'",updvals[i+1],endptr);
+                        break;
+                    }
 		    rate = pdp_new[i] / interval;		  
 		    break;
 		case DST_GAUGE:
-		    pdp_new[i] = atof(updvals[i+1]) * interval;
+                    errno = 0;
+		    pdp_new[i] = strtod(updvals[i+1],&endptr) * interval;
+                    if (errno > 0){
+                        rrd_set_error("converting  '%s' to float: %s",updvals[i+1],strerror(errno));
+                        break;
+                    };
+                    if (endptr[0] != '\0'){
+                        rrd_set_error("conversion of '%s' to float not complete: tail '%s'",updvals[i+1],endptr);
+                        break;
+                    }
 		    rate = pdp_new[i] / interval;		   
 		    break;
 		default:
